/*
 * Copyright (c) 2017, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * gc global variable provides access to GUI Composer infrastructure components and project information.
 * For more information, please see the Working with Javascript guide in the online help.
 */
var gc = gc || {};
gc.services = gc.services || {};

/*
*  Boilerplate code for creating computed data bindings
*/
document.addEventListener('gc-databind-ready', function() 
{
    /* 
	*   Add custom computed value databindings here, using the following method:
	*
    *   function gc.databind.registry.bind(targetBinding, modelBinding, [getter], [setter]);
	*
	*   param targetBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param modelBinding - single binding string or expression, or array of binding strings for multi-way binding.
	*   param getter - (optional) - custom getter function for computing the targetBinding value(s) based on modelBinding value(s).
	*   param setter - (optional) - custom setter function for computing the modelBinding value(s) based on targetBinding value(s).
    */
	
	// For example, a simple computed values based on simple expression
	// gc.databind.registry.bind('widget.id.propertyName', "targetVariable == 1 ? 'binding is one' : 'binding is not one'");
	
	// Or a custom two-way binding with custome getter and setter functions.  (setter is optional)  (getter only indicates one-way binding)
	// gc.databind.registry.bind('widget.id.propertyName', "targetVariable", function(value) { return value*5/9 + 32; }, function(value) { (return value-32)*9/5; });
	
	// Event 1 to n bindings
	/* 
    gc.databind.registry.bind('widget.date.value', 
        // dependant bindings needed in order to compute the date, in name/value pairs.
        {
            weekday: 'widget.dayOfWeek.selectedText',
            day: 'widget.dayOfMonth.value',
            month: 'widget.month.selectedText',
            year: 'widget.year.value'
        }, 
        // getter for date computation
        function(values) 
        {
            // compute and return the string value to bind to the widget with id 'date'
            return values.weekday + ', ' + values.month + ' ' + values.day + ', ' + values.year;
        }
    ); 
	*/
});

/*
*  Boilerplate code for creating custom actions
*/
document.addEventListener('gc-nav-ready', function() 
{
    /* 
	*   Add custom actions for menu items using the following api:
	*
    *   function gc.nav.registryAction(id, runable, [isAvailable], [isVisible]);
	*
	*   param id - uniquely identifies the action, and should correspond to the action property of the menuaction widget.
	*   param runable - function that performs the custom action.
	*   param isAvailable - (optional) - function called when the menu action is about to appear.  Return false to disable the action, or true to enable it.
	*   param isVisible - (optional) - function called when the menu action is about to appear.  Return false to hide the action, or true to make it visible.
    */
	
	// For example,
	// gc.nav.registerAction('myCustomCloseAction', function() { window.close(); }, function() { return true; }, function() { return true; });
	
	// Alternatively, to programmatically disable a menu action at any time use:
	// gc.nav.disableAction('myCustomCloseAction);    then enable it again using:  gc.nav.enableAction('myCustomCloseAction'); 
});

/*
*  Boilerplate code for working with components in the application gist
*/

var initComplete = false;
var templateObj;

// Wait for DOMContentLoaded event before trying to access the application template
var init = function() {
    templateObj = document.querySelector('#template_obj');

    // Wait for the template to fire a dom-change event to indicate that it has been 'stamped'
    // before trying to access components in the application.
	templateObj.addEventListener('dom-change',function(){
	    if (initComplete) return;
	    this.async(function(){
    	    initComplete = true;
    	    console.log("Application template has been stamped.");
  	        // Now that the template has been stamped, you can use 'automatic node finding' $ syntax to access widgets.
  	        // e.g. to access a widget with an id of 'widget_id' you can use templateObj.$.widgetId
  	        
  	        // Override default Software Manifest
            var aboutBox = document.querySelector('ti-widget-aboutbox');    
            aboutBox.addEventListener("aboutbox_opening", function(event){
                aboutBox.softwareManifestLink = "app/docs/MSP-EXP432P4111LaunchPadOOBGUI_1.0.0_manifest.html";
            });

            templateObj.$.ti_widget_container1.style.minWidth = "400px";
            templateObj.$.ti_widget_container2.style.minWidth = "400px";
            templateObj.$.ti_widget_container3.style.minWidth = "400px";
            templateObj.$.ti_widget_container1.style.minHeight = "600px";
            templateObj.$.ti_widget_container2.style.minHeight = "600px";
            templateObj.$.ti_widget_container3.style.minHeight = "600px";
            
            templateObj.$.ti_widget_label_colorwheel.style.minHeight = "54px";
  	        
  	        templateObj.$.ti_widget_container1.style.border = "0px";
  	        templateObj.$.ti_widget_container3.style.border = "0px";
  	        templateObj.$.ti_widget_container_colorsliders1.style.overflow = "hidden";

  	        /* LCD Controls */
  	        templateObj.$.ti_widget_input_lcd_string.children.paperInput.allowedPattern = "[a-zA-Z0-9 ]";
  	        templateObj.$.ti_widget_input_lcd_string.children.paperInput.style.width = "80%";
  	        templateObj.$.ti_widget_input_lcd_string.style.textAlign = "center";
            
            templateObj.$.ti_widget_button_scroll_string.onclick = function(){
                templateObj.$.my_usb_uart.sendValue({"lcdScrollString":templateObj.$.ti_widget_input_lcd_string.value.toUpperCase()});
            };
            
            templateObj.$.ti_widget_button_show_string.onclick = function(){
                templateObj.$.my_usb_uart.sendValue({"lcdShowString":templateObj.$.ti_widget_input_lcd_string.value.toUpperCase()});
            };
            
  	        /* Setup LCD symbol toggle buttons */
  	        templateObj.$.ti_widget_image_antenna.onclick = function(){
                if(this.imagePath == "app/img/antenna_white.svg") {
                    this.imagePath = "app/img/antenna_black.svg";
                    updateLCDMemory(42, 0x04, 0);
                    updateLCDMemory(46, 0x05, 0);
                }
                else {
                    this.imagePath = "app/img/antenna_white.svg";
                    updateLCDMemory(42, 0x04, 1);
                    updateLCDMemory(46, 0x05, 1);
                }
  	        }
  	        templateObj.$.ti_widget_image_antenna.style.cursor = "pointer";
  	        templateObj.$.ti_widget_image_antenna.children[0].style.width = "auto";
  	        
  	        templateObj.$.ti_widget_image_heart.onclick = function(){
                if(this.imagePath == "app/img/heart_white.svg") {
                    this.imagePath = "app/img/heart_black.svg";
                    updateLCDMemory(2, 0x04, 0);
                }
                else {
                    this.imagePath = "app/img/heart_white.svg";
                    updateLCDMemory(2, 0x04, 1);
                }
  	        }
  	        templateObj.$.ti_widget_image_heart.style.cursor = "pointer";
  	        templateObj.$.ti_widget_image_heart.children[0].style.width = "auto";
  	        
  	        templateObj.$.ti_widget_image_exclamation.onclick = function(){
                if(this.imagePath == "app/img/exclamation_white.svg") {
                    this.imagePath = "app/img/exclamation_black.svg";
                    updateLCDMemory(2, 0x01, 0);
                }
                else {
                    this.imagePath = "app/img/exclamation_white.svg";
                    updateLCDMemory(2, 0x01, 1);
                }
  	        }
  	        templateObj.$.ti_widget_image_exclamation.style.cursor = "pointer";
  	        templateObj.$.ti_widget_image_exclamation.children[0].style.width = "auto";
  	        
  	        templateObj.$.ti_widget_image_timer.onclick = function(){
                if(this.imagePath == "app/img/timer_white.svg") {
                    this.imagePath = "app/img/timer_black.svg";
                    updateLCDMemory(2, 0x08, 0);
                }
                else {
                    this.imagePath = "app/img/timer_white.svg";
                    updateLCDMemory(2, 0x08, 1);
                }
  	        }
  	        templateObj.$.ti_widget_image_timer.style.cursor = "pointer";
  	        templateObj.$.ti_widget_image_timer.children[0].style.width = "auto";
  	        
  	        templateObj.$.ti_widget_image_record.onclick = function(){
                if(this.imagePath == "app/img/record_white.svg") {
                    this.imagePath = "app/img/record_black.svg";
                    updateLCDMemory(2, 0x02, 0);
                }
                else {
                    this.imagePath = "app/img/record_white.svg";
                    updateLCDMemory(2, 0x02, 1);
                }
  	        }
  	        templateObj.$.ti_widget_image_record.style.cursor = "pointer";
  	        templateObj.$.ti_widget_image_record.children[0].style.width = "auto";
  	        
            
            templateObj.$.ti_widget_button_lcd_set.onclick = function(){
                var array = [
                    templateObj.$.ti_widget_numberbox_lcd_memory.value,
                    parseInt(templateObj.$.ti_widget_textbox_memory_data.value, 16),
                    0
                ];
                templateObj.$.my_usb_uart.sendValue({"lcdUpdateMemory":array});
            };
            
            templateObj.$.ti_widget_button_lcd_clear.onclick = function(){
                var array = [
                    templateObj.$.ti_widget_numberbox_lcd_memory.value,
                    parseInt(templateObj.$.ti_widget_textbox_memory_data.value, 16),
                    1
                ];
                templateObj.$.my_usb_uart.sendValue({"lcdUpdateMemory":array});
            };
            
            templateObj.$.ti_widget_button_lcd_toggle.onclick = function(){
                var array = [
                    templateObj.$.ti_widget_numberbox_lcd_memory.value,
                    parseInt(templateObj.$.ti_widget_textbox_memory_data.value, 16),
                    2
                ];
                templateObj.$.my_usb_uart.sendValue({"lcdUpdateMemory":array});
            };
            
            /* */
            templateObj.$.ti_widget_button_enable_animation.onclick = function(){
                templateObj.$.my_usb_uart.sendValue({"lcdStartAnimation":true});
                updateBatterySVG();
            };
            
            templateObj.$.ti_widget_button_disable_animation.onclick = function(){
                templateObj.$.my_usb_uart.sendValue({"lcdStartAnimation":false});
            };
            
            /* Setup Battery symbol */
            var batteryAnimationFrames = [0x11, 0x31, 0x33, 0x73, 0x77, 0xF7, 0xFF, 0xFF];
            
            function updateBatterySVG() {
                var frame = batteryAnimationFrames[templateObj.$.ti_widget_droplist_animation_frames.selectedIndex];
                
                for (var i = 0; i < 8; i++)
                {
                    switch (i){
                        case 0:
                            if (frame & (0x01 << i))
                                document.getElementById("battery_top").style.fill = "black";
                            else
                                document.getElementById("battery_top").style.fill = "white";
                            break;
                        case 1:
                            if (frame & (0x01 << i))
                                document.getElementById("battery_charge_2").style.fill = "black";
                            else
                                document.getElementById("battery_charge_2").style.fill = "white";
                            break;
                        case 2:
                            if (frame & (0x01 << i))
                                document.getElementById("battery_charge_4").style.fill = "black";
                            else
                                document.getElementById("battery_charge_4").style.fill = "white";
                            break;
                        case 3:
                            if (frame & (0x01 << i))
                                document.getElementById("battery_charge_6").style.fill = "black";
                            else
                                document.getElementById("battery_charge_6").style.fill = "white";
                            break;
                        case 4:
                            if (frame & (0x01 << i)) {
                                document.getElementById("battery_bracket_left").style.fill = "black";
                                document.getElementById("battery_bracket_right").style.fill = "black";
                            }
                            else {
                                document.getElementById("battery_bracket_left").style.fill = "white";
                                document.getElementById("battery_bracket_right").style.fill = "white";
                            }
                            break;
                        case 5:
                            if (frame & (0x01 << i))
                                document.getElementById("battery_charge_1").style.fill = "black";
                            else
                                document.getElementById("battery_charge_1").style.fill = "white";
                            break;
                        case 6:
                            if (frame & (0x01 << i))
                                document.getElementById("battery_charge_3").style.fill = "black";
                            else
                                document.getElementById("battery_charge_3").style.fill = "white";
                            break;
                        case 7:
                            if (frame & (0x01 << i))
                                document.getElementById("battery_charge_5").style.fill = "black";
                            else
                                document.getElementById("battery_charge_5").style.fill = "white";
                            break;
                    }
                    
                    updateLCDAnimationMemory(i, batteryAnimationFrames[i], 0);
                    updateLCDAnimationMemory(i, ~(batteryAnimationFrames[i]), 1);
                }                    
            }
            
            var xhr = new XMLHttpRequest();
            xhr.open('get','app/img/battery.svg',true);
            xhr.onreadystatechange = function(){
                if (xhr.readyState != 4) return;
                var svg = xhr.responseXML.documentElement;
                svg = document.importNode(svg,true); // surprisingly optional in these browsers
                templateObj.$.ti_widget_container_battery_symbol.appendChild(svg);
                
                document.getElementById("svg_battery").style.height = "100%";
                document.getElementById("svg_battery").style.width = "100%";
                document.getElementById("battery_bracket_left").style.cursor = "pointer";
                document.getElementById("battery_bracket_right").style.cursor = "pointer";
                document.getElementById("battery_top").style.cursor = "pointer";
                document.getElementById("battery_charge_1").style.cursor = "pointer";
                document.getElementById("battery_charge_2").style.cursor = "pointer";
                document.getElementById("battery_charge_3").style.cursor = "pointer";
                document.getElementById("battery_charge_4").style.cursor = "pointer";
                document.getElementById("battery_charge_5").style.cursor = "pointer";
                document.getElementById("battery_charge_6").style.cursor = "pointer";
                
                updateBatterySVG();
                
                templateObj.$.ti_widget_droplist_animation_frames.onchange = updateBatterySVG;

                document.getElementById("battery_bracket_left").onclick = function() {
                    batteryAnimationFrames[templateObj.$.ti_widget_droplist_animation_frames.selectedIndex] ^= 0x10;
                    updateBatterySVG();
                }
                
                document.getElementById("battery_bracket_right").onclick = function() {
                    batteryAnimationFrames[templateObj.$.ti_widget_droplist_animation_frames.selectedIndex] ^= 0x10;
                    updateBatterySVG();
                }
                
                document.getElementById("battery_top").onclick = function() {
                    batteryAnimationFrames[templateObj.$.ti_widget_droplist_animation_frames.selectedIndex] ^= 0x01;
                    updateBatterySVG();
                }
                
                document.getElementById("battery_charge_1").onclick = function() {
                    batteryAnimationFrames[templateObj.$.ti_widget_droplist_animation_frames.selectedIndex] ^= 0x20;
                    updateBatterySVG();
                }
                
                document.getElementById("battery_charge_2").onclick = function() {
                    batteryAnimationFrames[templateObj.$.ti_widget_droplist_animation_frames.selectedIndex] ^= 0x02;
                    updateBatterySVG();
                }
                
                document.getElementById("battery_charge_3").onclick = function() {
                    batteryAnimationFrames[templateObj.$.ti_widget_droplist_animation_frames.selectedIndex] ^= 0x40;
                    updateBatterySVG();
                }
                
                document.getElementById("battery_charge_4").onclick = function() {
                    batteryAnimationFrames[templateObj.$.ti_widget_droplist_animation_frames.selectedIndex] ^= 0x04;
                    updateBatterySVG();
                }
                
                document.getElementById("battery_charge_5").onclick = function() {
                    batteryAnimationFrames[templateObj.$.ti_widget_droplist_animation_frames.selectedIndex] ^= 0x80;
                    updateBatterySVG();
                }
                
                document.getElementById("battery_charge_6").onclick = function() {
                    batteryAnimationFrames[templateObj.$.ti_widget_droplist_animation_frames.selectedIndex] ^= 0x08;
                    updateBatterySVG();
                }
            };
            xhr.send();

            /* Setup Colorwheel */
            templateObj.$.ti_widget_container_colorwheel.style.overflow = "hidden";
            
            var canv = document.createElement('canvas');
            templateObj.$.ti_widget_container_colorwheel.appendChild(canv);
            
            canv.id = 'myCanvas';
            canv.style.width = "100%";
            canv.style.height = "100%";
            canv.style.flex = "1";
            
            var canv1 = document.createElement('canvas');
            templateObj.$.ti_widget_container_colorwheel.appendChild(canv1);
            canv1.id = 'myCanvas1';
            canv1.style.position = "absolute";
            canv1.style.top = "0px";
            canv1.style.left = "0px";
            
            var ctx = canv.getContext('2d');
            var ctx1 = canv1.getContext('2d');
            var p, colorWheelRadius;
            
            window.addEventListener('resize', function() {
                canv.width = 0;
                canv.height = 0;
                canv.style.width = "1px";
                canv.style.height = "1px";
            
                setTimeout(drawColorWheel, 0);
            });
            
            function drawColorWheel() {
                // Draw Color Wheel
                canv.style.width = "100%";
                canv.style.height = "100%";
                canv.width = canv.offsetWidth;
                canv.height = canv.offsetHeight;
                canv1.width = canv.width;
                canv1.height = canv.height;
                
                p = {
                    x: canv.offsetWidth / 2,
                    y: canv.offsetHeight / 2
                };
                
                ctx.translate(p.x, p.y);
                ctx1.translate(p.x, p.y);
                
                colorWheelRadius = (260/300) * (p.x < p.y ? p.x : p.y);
                
                var thickness = (255/260) * colorWheelRadius,
                    start = Math.PI,
                    end = start + Math.PI * 2,
                    step = Math.PI / 360,
                    ang = 0,
                    grad,
                    r = 0,
                    g = 0,
                    b = 0,
                    pct = 0;
    
                ctx.clearRect(0 - p.x, 0 - p.y, canv.width, canv.height);
    
                //Color wheel edge (black)
                ctx.beginPath();
                ctx.arc(0, 0, colorWheelRadius + ((5/260) * colorWheelRadius), 0, 2 * Math.PI);
                ctx.fillStyle = 'white';
                ctx.fill();
    
                //Color wheel gradient
                for (ang = start; ang <= end; ang += step) {
                    ctx.save();
                    ctx.rotate(-ang);
                    grad = ctx.createLinearGradient(0, colorWheelRadius - thickness, 0, colorWheelRadius);
                    grad.addColorStop(1, 'white');
    
                    var hue = 360 - (ang - start) / (end - start) * 360;
                    var saturation = '100%';
                    var light = '50%';
    
                    grad.addColorStop(.5, 'hsl(' + [hue, saturation, light].join() + ')');
                    grad.addColorStop(0, 'black');
                    ctx.fillStyle = grad;
    
                    ctx.fillRect(0, colorWheelRadius - thickness, 5, thickness);
                    ctx.restore();
                }
    
                //Color wheel center (white)
                ctx.beginPath();
                ctx.arc(0, 0, colorWheelRadius - thickness, 0, 2 * Math.PI);
                ctx.fillStyle = 'black';
                ctx.fill();
            }
            
            function getPosition(event) {
                var x = event.clientX;
                var y = event.clientY;
    
                var box = canv.getBoundingClientRect();
    
                x -= box.left;
                y -= box.top;
                
                selectColor(x, y);
    
                //console.log(x+", " + y);
                // var clickRadius = (Math.sqrt(Math.pow(p.x - x, 2) + Math.pow(p.y - y, 2)));
    
                // if (clickRadius <= (265/300) * (p.x < p.y ? p.x : p.y)) {
                //     var rgba = ctx.getImageData(x, y, 1, 1).data;
                //     selectColor(x - p.x, y - p.y, rgba);
                    
                //     templateObj.$.ti_widget_slider_red.value = rgba[0];
                //     templateObj.$.ti_widget_slider_green.value = rgba[1];
                //     templateObj.$.ti_widget_slider_blue.value = rgba[2];
                //     templateObj.$.my_usb_uart.sendValue(
                //         {
                //             "red": templateObj.$.ti_widget_slider_red.value,
                //             "blue": templateObj.$.ti_widget_slider_blue.value,
                //             "green": templateObj.$.ti_widget_slider_green.value,
                //             "bpm": templateObj.$.ti_widget_spinner_bpm.value
                //         });
                // }
                event.preventDefault();
            }
            
            function selectColor(x, y) {
                var clickRadius = (Math.sqrt(Math.pow(p.x - x, 2) + Math.pow(p.y - y, 2)));
                
                var boundedX, boundedY;
                if (clickRadius <= colorWheelRadius) {
                    boundedX = x;
                    boundedY = y;
                }
                else {
                    boundedX = p.x + ((x - p.x) * (colorWheelRadius/clickRadius));
                    boundedY = p.y + ((y - p.y) * (colorWheelRadius/clickRadius));
                }
                
                    var rgba = ctx.getImageData(boundedX, boundedY, 1, 1).data;
                    
                    // Draw color selection circle
                    ctx1.clearRect(0-p.x, 0-p.y, canv1.width, canv1.height);
                    ctx1.beginPath();
                    ctx1.arc(boundedX - p.x, boundedY - p.y, 5, 0, 2 * Math.PI);
                    ctx1.lineWidth = 4;
                    ctx1.strokeStyle = "rgb(" + [255 - rgba[0], 255 - rgba[1], 255 - rgba[2]] + ")";
                    ctx1.stroke();
                    
                    var temp1, temp2, temp3;
                    temp1 = templateObj.$.ti_widget_slider_red.children[0].children[1]._immediateValueChanged;
                    temp2 = templateObj.$.ti_widget_slider_green.children[0].children[1]._immediateValueChanged;
                    temp3 = templateObj.$.ti_widget_slider_blue.children[0].children[1]._immediateValueChanged;
                    templateObj.$.ti_widget_slider_red.children[0].children[1]._immediateValueChanged = function(){};
                    templateObj.$.ti_widget_slider_green.children[0].children[1]._immediateValueChanged = function(){};
                    templateObj.$.ti_widget_slider_blue.children[0].children[1]._immediateValueChanged = function(){};
                    templateObj.$.ti_widget_slider_red.children[0].children[1].value = rgba[0];
                    templateObj.$.ti_widget_slider_green.children[0].children[1].value = rgba[1];
                    templateObj.$.ti_widget_slider_blue.children[0].children[1].value = rgba[2];
                    updateLEDColorBpm();
                    templateObj.$.ti_widget_slider_red.children[0].children[1]._immediateValueChanged = temp1;
                    templateObj.$.ti_widget_slider_green.children[0].children[1]._immediateValueChanged = temp2;
                    templateObj.$.ti_widget_slider_blue.children[0].children[1]._immediateValueChanged = temp3;
            }
            
            function mDown(event) {
                getPosition(event);
                canv1.addEventListener("mousemove", getPosition, false);
                event.preventDefault();
            }
    
            function mUp(event) {
                canv1.removeEventListener("mousemove", getPosition, false);
                event.preventDefault();
            }
            
            templateObj.$.ti_widget_slider_red.children[0].children[1]._immediateValueChanged = function() {
                updateLEDColorBpm();
            };
            templateObj.$.ti_widget_slider_green.children[0].children[1]._immediateValueChanged = function() {
                updateLEDColorBpm();
            };
            templateObj.$.ti_widget_slider_blue.children[0].children[1]._immediateValueChanged = function() {
                updateLEDColorBpm();
            };
            
            drawColorWheel();
            selectColor(p.x, p.y);
            updateLEDColorBpm();
            canv1.addEventListener("mousedown", mDown, false);
            canv1.addEventListener("mouseup", mUp, false);
        },1);

	});
};

templateObj = document.querySelector('#template_obj');
if (templateObj) {
    init();
} else {
    document.addEventListener('DOMContentLoaded',init.bind(this))
}

function updateLCDMemory(lcdMemory, byte, operation){
    var array = [
        lcdMemory,
        byte,
        operation
    ];
    templateObj.$.my_usb_uart.sendValue({"lcdUpdateMemory":array});
}

function updateLCDAnimationMemory(lcdMemory, byte, operation){
    var array = [
        lcdMemory,
        byte,
        operation
    ];
    templateObj.$.my_usb_uart.sendValue({"lcdUpdateAnimationMemory":array});
}

function updateLEDColorBpm(){
    var r = templateObj.$.ti_widget_slider_red.value;
    var g = templateObj.$.ti_widget_slider_green.value;
    var b = templateObj.$.ti_widget_slider_blue.value;
    templateObj.$.ti_widget_container_selected_color.style.backgroundColor = "rgb(" + [r, g, b].join(',') +")";
    templateObj.$.ti_widget_label_selected_color.label = rgbToHex(r, g, b).toUpperCase();
    templateObj.$.ti_widget_label_selected_color.fontColor = "rgb(" + [255 - r, 255 - g, 255 - b] + ")";
    templateObj.$.my_usb_uart.sendValue(
        {
            "red": templateObj.$.ti_widget_slider_red.value,
            "blue": templateObj.$.ti_widget_slider_blue.value,
            "green": templateObj.$.ti_widget_slider_green.value,
            "bpm": templateObj.$.ti_widget_spinner_bpm.value
        });
}

function componentToHex(c) {
    var hex = c.toString(16);
    return hex.length == 1 ? "0" + hex : hex;
}

function rgbToHex(r, g, b) {
    return "#" + componentToHex(r) + componentToHex(g) + componentToHex(b);
}